use axum::http::StatusCode;
use serde::Serialize;

#[derive(Serialize)]
pub struct ErrorResponse {
    pub id: String,
    pub error: String,
}

pub trait HttpStatus {
    fn status_code(&self) -> StatusCode;
}

#[macro_export]
macro_rules! http_error {
    ($error_type:ty) => {
        impl axum::response::IntoResponse for $error_type {
            fn into_response(self) -> axum::response::Response {
                let status = self.status_code();
                let id = uuid::Uuid::new_v4().to_string();
                tracing::error!("{}: {}", &id, snafu::Report::from_error(&self));

                let error_response = $crate::api::error::ErrorResponse {
                    id,
                    error: self.to_string(),
                };

                (status, axum::Json(error_response)).into_response()
            }
        }
    };
}
